#--- Top Matter ---------------------------------------------------------------#

library(shinyjs)

# Linking coefficients data
lc <- read.csv("./data/linking_coefficients.csv", header = T)

#--- User Interface (UI) ------------------------------------------------------#

ui <- fluidPage(theme = "spacelab.css",
  tags$head(
    tags$link(rel = "stylesheet", type = "text/css", href = "css_hacks.css")
  ),
  titlePanel(HTML(paste0("Linking PROMIS", tags$sup("\U00AE"), " Pediatric and Adult Physical Function Banks"))),
  sidebarLayout(
    # Side Panel
    sidebarPanel(
      fileInput("file", "Choose Dataset to Upload (.csv only)",
                multiple = FALSE,
                accept = c("text/csv",
                           "text/comma-separated-values,text/plain",
                           ".csv")),
      uiOutput("link_var"),
      selectInput('link_type', 'Type of Linkage',
                  choices = list(
                    "Pediatric to Adult" = c(
                      "Pediatric MOBILITY to Adult PHYSICAL FUNCTION" = "p_to_a_mob",
                      "Pediatric UPPER EXTREMITY to Adult PHYSICAL FUNCTION" = "p_to_a_uppex",
                      "Pediatric FATIGUE to Adult FATIGUE" = "p_to_a_fat",
                      "Pediatric PAIN to Adult PAIN" = "p_to_a_pain"),
                    "Adult to Pediatric" = c(
                      "Adult PHYSICAL FUNCTION to Pediatric MOBILITY" = "a_to_p_mob",
                      "Adult UPPER EXTREMITY to Pediatric UPPER EXTREMITY" = "a_to_p_uppex",
                      "Adult FATIGUE to Pediatric FATIGUE" = "a_to_p_fat",
                      "Adult PAIN to Pediatric PAIN" = "a_to_p_pain")
                  ),
                  selectize = TRUE),
      textInput("new_name", "New Linked Variable Name", value = ""),
      actionButton("link", "Compute Linkage", width = "100%"),
      downloadButton("export", "Export Data"),
      actionButton("reset", "Start Over", width = "100%")
    ),
    # Main Panel
    mainPanel(
      withTags(
        tabsetPanel(id = "tabset_panel",
          tabPanel(title = "Background", value = "background",
            p(style = "padding-top:20px;",
              HTML(paste0("This application computes score linkages between
                          Pediatric and Adult versions of the Patient-Reported Outcomes Measurement Information System (PROMIS",
                          sup("\U00AE"), ") Physical Function item banks, as
                          described in:"))),
            p(HTML(paste0("Tulsky, D. S., Kisala, P., Boulton, A. J., Jette, A.,
                          Thissen, D., Ni, P., ...Reeve, B. (in press).
                          Determining a Transitional Scoring Link between PROMIS",
                          sup("\U00AE"), " Pediatric and Adult Physical
                          Health Measures.", em(" Quality of Life Research.")))),
            p("In this study, scores were linked via calibrated projection
              (Thissen et al., 2011; Thissen, Liu, Magnus, & Quinn, 2015),
              an item response theory (IRT)-based linking method used to
              predict scores on one measure (e.g., PROMIS Adult Fatigue) from
              another measure (e.g., PROMIS Pediatric Fatigue). Such linkages are
              useful, for example, in longitudinal studies where developmental
              transitions occur (e.g., transition to adulthood) or when comparing two
              samples that completed different measures of the same (or similar)
              constructs."),
            p("Instructions for this application can be found under the
              'Instructions' tab. For more information on PROMIS, please consult the",
              a(href="http://www.healthmeasures.net/index.php", target="_blank", "HealthMeasures"), " website"), 
            p("Users are advised that error is inevitably
              introduced when linking methods are used, a consequence of having to
              predict scores on a measure instead of directly administered the
              intended measure; therefore, linkages are best applied ad hoc and
              when a research design offers no alternative (Reeve et al., 2016)."),
            b("References"),
            p(style = "padding-top:10px;", "Reeve, B. B., Thissen, D., DeWalt, D. A., Huang, I. C., Liu, Y., 
              Magnus, B., ..., Tulsky, D. S. (2016). Linkage between the PROMIS 
              pediatric and adult emotional distress measures. ", em("Quality of Life Research, 25(4)", "823-833.")),
            p("Thissen, D., Varni, J. W., Stucky, B. D., Liu, Y., Irwin, D. E., &
              Dewalt, D. A. (2011). Using the PedsQL 3.0 asthma module to obtain scores comparable with those of the PROMIS pediatric asthma impact scale (PAIS). ", tags$em("Qual Life Res"), "20(9), 1497-1505."),
            p("Thissen, D., Liu, Y., Magnus, B., & Quinn, H. (2015). Extending the
              use of multidimensional IRT calibration as projection: Many-to-one
              linking and linear computation of projected scores. In L. A. van der
              ark, D. M. Bolt, W.-C. Wang, J. A. Doublas & S.-M. Chow (Eds.), ",
              em("Quantitative Psychology Research: The 79th annual meeting of the
                 psychometric society, Madison, Wisconsin, 2014"), "(pp. 1-16).
              New York: Springer.")),
          tabPanel(title = "Instructions", value = "instructions",
            p(style = "padding-top:20px;", "To use the application, follow
              the steps below."),
            p(b("Step 1: Upload dataset."), " Click on the 'Browse' button in
              the left-hand column and upload a dataset (in .csv format)
              containing the variables to be linked. Once uploaded, the dataset will appear in the
              'Dataset' tab"),
            p(b("Step 2: Compute linkages."), " For each linkage, select a
              variable in the dataset (via the 'Select Variable in Dataset to
              Link' drop down menu) containing measure scores from which scores
              will be linked. Next, choose the linkage to be computed under the
              'Type of Linkage' drop down menu and type in the name of the
              variable to be created in the 'New Linked Variable Name' dialog
              box; if a name is not provided, a default label will be
              assigned. Finally, press the 'Compute Linkage' button; a new
              variable with the linked scores will appear in the dataset displayed."),
            p("To start over with the original dataset, thereby removing any
              newly created linkage variables, press the 'Start Over' button"),
            p(b("Step 3: Export dataset."), " Once all desired linkages have
              been computed, press the 'Export Dataset' button to download a
              copy of the dataset displayed in the 'Dataset' tab that contains
              the new linkage variables. This dataset will appear in the user's
              system download directory."),
            br(),
            p("Questions or feedback? Please contact Aaron Boulton at
              aboulton at udel.edu")
          ),
          tabPanel(title = "Dataset", value = "dataset",
            uiOutput("dataset_tab")
        ))
      )
    )
  )
)

#--- Define server logic ------------------------------------------------------#

server <- function(input, output, session) {
  
  # Create reactiveValues object 'values'
  values <- reactiveValues(dataset = NULL)
  
  # Store data 
  observeEvent(input$file, {
    values$dataset <- read.csv(input$file$datapath)
    updateTabsetPanel(session, "tabset_panel", selected = "dataset")
  })
  
  # Render data table for display
  output$dataset <- DT::renderDataTable({
    req(input$file)
    values$dataset
  },
  options = list(scrollX = TRUE))
  
  output$dataset_tab <- renderUI({
    if(is.null(input$file)) {
      p(style = "padding-top:20px;", "Dataset not loaded")
    } else {
      wellPanel(
        DT::dataTableOutput("dataset")   
      )
    }
  })
    
  # Create linkage
  observeEvent(input$link, {
    if (input$new_name != "") {
      new_name <- input$new_name
    } else {
      new_name <- paste0("linkage_", input$link)
    }
    int <- lc[lc$linkage == input$link_type, "int"]
    slope <- lc[lc$linkage == input$link_type, "slope"]
    linked_var <- int + values$dataset[, input$var] * slope
    out <- cbind.data.frame(new_name = linked_var, values$dataset)
    names(out)[names(out) == "new_name"] <- new_name
    values$dataset <- out
  })
  
  # Display variable names
  output$link_var <- renderUI({
    if (is.null(input$file)) {
      selectInput('var', 'Select Variable in Dataset to Link',
                  choices = "", selectize = FALSE)
    } else {
      selectInput('var', 'Select Variable in Dataset to Link',
                  c(colnames(values$dataset)), selectize = FALSE)
      
    }
  })
  
  # Export Dataset csv of selected dataset ----
  output$export <- downloadHandler(
    filename = function() {
      paste("dataset_with_linkages.csv", sep = "")
    },
    content = function(file) {
      write.csv(values$dataset, file, row.names = FALSE)
    }
  )
  
  # Start over
  observeEvent(input$reset, {
    values$dataset <- read.csv(input$file$datapath)
    updateTabsetPanel(session, "tabset_panel", selected = "dataset")
  })
}

#--- Run the app --------------------------------------------------------------#
shinyApp(ui = ui, server = server)